#!/usr/bin/env python3
"""utility to move/delete duplicate files of the same size and context in specified folder.

command string parameters:
first parameter:    full_path_to_folder - folder where duplicates are searched.
second parameter:   optional - folder for storage duplicate files """

import argparse
import logging
import pathlib
import sys
import remove_duplicates.my_utils as my_utils


def recursive_process_folder(start_folder: str, trash_folder: str, file_name_pattern: list):
    """
    :param start_folder: Search for duplicate files starts from this folder.
    :param trash_folder: found copies of files are transferred to this folder.
    :param file_name_pattern: Only files matching the pattern are processed.
    :return: count file copies deleted/moved.
    """
    ret_val = 0
    try:
        ret_val = my_utils.delete_duplicate_file(start_folder, file_name_pattern, trash_folder, logging)
    except PermissionError as ex:
        logging.warning(f"Folder {start_folder}. OS Error code: {ex.errno}. Error message: {ex.strerror}!")
    else:
        logging.info(f"Folder {start_folder} processed. Found {ret_val} copies!")

    # enumerating
    pth = pathlib.Path(start_folder)
    for child in pth.iterdir():
        try:
            if child.is_dir():
                ret_val += recursive_process_folder(str(child.resolve()), trash_folder, file_name_pattern)
        except PermissionError:
            folder_name = str(child.resolve())
            logging.warning(f"Access is denied! Folder: {folder_name}")
    # return value
    return ret_val


def main() -> int:
    """return: count file copies deleted/moved
    if error return my_utils.INVALID_VALUE."""
    str_storage_folder = None  # default value
    log_file_name = None
    str_search_folder = my_utils.get_folder_name_from_path(sys.argv[0])  # default value

    parser = argparse.ArgumentParser(description="""Utility to recursive search and move/delete duplicate files 
                                                of the same size and context in specified folder.""",
                                     epilog="""If the storage folder is not specified, 
                                                then duplicate files will be deleted!
                                                If the number of command line parameters is zero, 
                                                then the search folder = current folder.""")

    parser.add_argument("--start_folder", type=str, help="The folder with which the recursive search begins.")
    parser.add_argument("--recycle_bin", type=str, help="Folder for storing duplicate files.")
    parser.add_argument("--log_file", type=str, help="Log file name.")
    parser.add_argument("--fn_pattern", type=str,
                        help="File name pattern. Only files matching the pattern are processed! "
                             "Provides support for Unix shell-style wildcards.", default="*.*")

    args = parser.parse_args()

    fn_pattern = ""
    if args.fn_pattern:
        fn_pattern = args.fn_pattern.replace(" ", "")  # удаляю все пробелы из строки
        fn_pattern = fn_pattern.split(",")  # создаю список

    if args.log_file:
        log_file_name = args.log_file
    # setup logger start
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.INFO)

    # Если пользователь не задал имя файла-журнала, поэтому журналом становится sys.stdout
    handler = logging.StreamHandler(sys.stdout)
    if log_file_name:
        handler = logging.FileHandler(log_file_name, "w", "utf-8")

    formatter = logging.Formatter("%(asctime)s: %(levelname)s: %(message)s")
    handler.setFormatter(formatter)  # Pass handler as a parameter, not assign
    root_logger.addHandler(handler)
    # setup logger end

    if args.start_folder:
        str_search_folder = args.start_folder
        if not my_utils.is_folder_exist(str_search_folder):
            logging.critical(f"Invalid path to search folder: {str_search_folder}. Exit!")
            return my_utils.INVALID_VALUE

    if args.recycle_bin:
        str_storage_folder = args.recycle_bin
        if not my_utils.is_folder_exist(args.recycle_bin):
            logging.critical(f"Invalid path to storage folder: {args.recycle_bin}. Exit!")
            return my_utils.INVALID_VALUE

    # START
    logging.info(f"Search for duplicate files in the folder: {str_search_folder}")
    logging.info(f"Pattern file name: {fn_pattern}")
    if log_file_name:
        logging.info(f"Log file name: {log_file_name}")
    if str_storage_folder:
        logging.info(f"Storage folder: {str_storage_folder}")

    ret_val = recursive_process_folder(str_search_folder, str_storage_folder, fn_pattern)

    action = "deleted"
    if args.recycle_bin:
        action = "moved"

    logging.info(f"Total found {ret_val} copies of files.")

    if ret_val:
        logging.info(f"{ret_val} copies of files have been {action}.")

    return ret_val


if __name__ == "__main__":
    sys.exit(main())



import pathlib
import shutil
from operator import itemgetter
import hashlib
import os
import fnmatch

INVALID_VALUE = -1
NO_ERROR_VALUE = 0

__name__ = "my_utils"
__version__ = 1.1


def get_folder_name_from_path(strfullpathtofile: str) -> str:
    """ return folder name from full path file name """
    mypath = pathlib.Path(strfullpathtofile)
    if mypath.is_dir():
        return str(mypath.resolve())
    if mypath.is_file():
        return str(mypath.parent.resolve())
    raise ValueError(f"Invalid value: {strfullpathtofile}")


def get_folder_files_info(str_full_folder_path_name: str, file_name_pattern: list) -> [tuple, None]:
    """Return list of files in folder str_full_folder_path_name.
    Each list item contains a tuple of two elements (filename_without_path, file_size_in_bytes).
    Returned tuple sorted by file_size ascending.
    Only files matching the pattern file_name_pattern are included in the list!"""
    # flist = None
    lpath = pathlib.Path(str_full_folder_path_name)
    if not lpath.is_dir():
        return None  # return None if str_full_folder_path_name not folder

    # enumerating files ONLY!!!
    # lst_files = [(child.name, pathlib.Path(child).stat().st_size)
    #              for child in lpath.iterdir() if child.is_file() and fnmatch.fnmatch(child.name, file_name_pattern)]
    lst_files = list()
    for child in lpath.iterdir():
        if child.is_file():
            for pattern in file_name_pattern:
                if fnmatch.fnmatch(child.name, pattern):
                    lst_files.append((child.name, pathlib.Path(child).stat().st_size))
                    break

    return tuple(sorted(lst_files, key=itemgetter(1)))


def get_hash_file(path: str, algorithm="md5", bufsize=4096) -> bytes:
    """return hash of file"""
    h = hashlib.new(algorithm)
    with open(path, "rb") as f:
        for chunk in iter(lambda: f.read(bufsize), b""):
            h.update(chunk)

    return h.digest()


def is_folder_exist(full_folder_path: str) -> bool:
    """check folder for exist and is folder
    return value is Boolean!"""
    folder = pathlib.Path(full_folder_path)
    return folder.is_dir() and folder.exists()


def get_full_file_name(str_folder_owner: str, str_file_name: str) -> str:
    """returns the full file name adding the folder name and file name"""
    return str_folder_owner + os.path.sep + str_file_name


def delete_duplicate_file(folder_full_path: str, file_name_pattern: list,
                          storage_folder: str = None, log=None) -> int:
    """move/delete duplicate files of the same size and context in specified folder (folder_full_path).

    folder_full_path - folder where duplicates are searched.
    storage_folder - optional - folder for storage duplicate files
    If the storage_folder is not specified, then duplicate files will be deleted!

    in case of error returns a negative value.
    if successful, returns the number of found duplicate files subjected to move / delete
    """
    ret_val = INVALID_VALUE

    if not is_folder_exist(folder_full_path):
        return ret_val

    # START
    if storage_folder and not is_folder_exist(storage_folder):
        return ret_val

    file_list = get_folder_files_info(folder_full_path, file_name_pattern)

    if not file_list:  # zero files for compare. exit
        return NO_ERROR_VALUE

    tpl_first = None

    index_file_name, index_file_size = range(2)

    ret_val = NO_ERROR_VALUE

    for item in file_list:
        if tpl_first is None:
            tpl_first = item
            continue
        if item[index_file_size] == tpl_first[index_file_size]:
            fname0 = get_full_file_name(folder_full_path, item[index_file_name])
            fname1 = get_full_file_name(folder_full_path, tpl_first[index_file_name])
            h0 = get_hash_file(fname0)
            h1 = get_hash_file(fname1)
            if h0 == h1:
                if not storage_folder:
                    f = pathlib.Path(fname0)
                    f.unlink()  # delete file
                    log.warning(f"File {fname0} deleted!")
                else:
                    dst = get_full_file_name(storage_folder, item[index_file_name])  # make full file name
                    # move duplicate file to storage folder
                    shutil.move(src=fname0, dst=dst, copy_function=shutil.copy)
                    log.warning(f"File {fname0} moved!")
                ret_val += 1    # для if и elseS
        else:  # file size not equals
            tpl_first = item
            continue

    return ret_val



#!/usr/bin/env python
import setuptools

with open("README.md") as file:
    read_me_description = file.read()

setuptools.setup(
    name="remove_duplicates",
    version="1.0.2",
    author="Roman Shevchik",
    author_email="goctaprog@gmail.com",
    description="Utility to recursive search and move/delete duplicate files of the same size and context in specified folder.",
    long_description=read_me_description,
    long_description_content_type="text/markdown",
    url="https://github.com/octaprog7/remove_duplicates",
    packages = setuptools.find_packages(where='src'),
    package_dir={"": "src"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
    ],
    python_requires='>=3.7.2,<3.12',
    entry_points={
        'console_scripts': [
            'rmdup = remove_duplicates.remove_dup:main',
        ],
    }
)
